def handleArrays(gribArr):
    import numpy as np
    from scipy.ndimage import zoom

    gribArr = zoom(gribArr, 2, order=1)
    oshape = gribArr.shape
    fixGrib = np.hstack((gribArr[:, oshape[1] / 2 + 1:oshape[1]],gribArr[:, 0:oshape[1] / 2 + 1]))

    return fixGrib

def updateBoundsAffine(inAffine):
    from rasterio import Affine, coords

    bounds = coords.BoundingBox(
        inAffine.c - 180.0 + (inAffine.a / 2.0),
        -inAffine.f,
        -(inAffine.c - 180.0 + (inAffine.a / 2.0)),
        inAffine.f)

    outAffine = Affine(inAffine.a / 2.0, inAffine.b,inAffine.c - 180.0 + (inAffine.a / 2.0),
         inAffine.d,inAffine.e / 2.0, inAffine.f)

    return outAffine

def loadRasterInfo(inputRaster):
    import rasterio
    with rasterio.open(inputRaster, 'r') as src:
        return {
            'shape': src.shape,
            'affine': src.affine,
            'dtype': src.meta['dtype'],
            'crs': src.crs,
            'kwargs': src.meta.copy()
        }

def getSnapDims(rasInfo):
    snapRows = max(list(
        i['shape'][0] for i in rasInfo
    ))

    snapCols = max(list(
        i['shape'][1] for i in rasInfo
    ))

    return (snapRows, snapCols)

def getSnapAffine(rasInfo, snapshape):
    rasMap = {i['shape']: {
        'affine': updateBoundsAffine(i['affine']),
        'dtype': i['dtype'],
        'crs': i['crs']
        } for i in rasInfo}
    return rasMap[snapshape]

def makeKwargs(bandNos, sMeta, sShape, zoomfactor):
    return {
        'driver': 'GTiff',
        'count': len(bandNos),
        'dtype': sMeta['dtype'],
        'height': sShape[0] * zoomfactor,
        'width': sShape[1] * zoomfactor,
        'transform': sMeta['affine'],
        'crs': sMeta['crs']
    }

def handleBands(data, snapshape):
    import numpy as np
    from scipy.ndimage import zoom
    try:
        data[np.where(data.mask == True)] = data.min()
    except AttributeError:
        pass
    if data.shape != snapshape:
        data = handleArrays(data)
        data = zoom(data, 2 * snapshape[1] / data.shape[1], order=1)
        data = ((np.roll(data, 1, axis=0) + data) / 2)[1:]

    else:
        data = handleArrays(data)

    return data


def loadBands(inputRaster, snapshape, gfs):
    import rasterio
    with rasterio.drivers():
        with rasterio.open(inputRaster, 'r') as src:
            if gfs:
                return list(handleBands(src.read_band(i), snapshape) for i in range(1, src.count + 1))
            else:
                return list(src.read())


import gribdoctor
import click, json, numpy as np

def upwrap_raster(inputRaster, outputRaster, bidx, bandtags):
    import rasterio

    with rasterio.drivers():
        with rasterio.open(inputRaster, 'r') as src:
            if bidx == 'all':
                bandNos = np.arange(src.count) + 1
            else:
                bandNos = list(int(i.replace(' ', '')) for i in bidx.split(','))

            fixedArrays = list(gribdoctor.handleArrays(src.read_band(i)) for i in bandNos)

            fixAff = gribdoctor.updateBoundsAffine(src.affine)
            if bandtags:
                tags = list(src.tags(i + 1) for i in range(src.count))
                click.echo(json.dumps(tags, indent=2))

        with rasterio.open(outputRaster, 'w',
            driver='GTiff',
            count=len(bandNos),
            dtype=src.meta['dtype'],
            height=src.shape[0] * 2,
            width=src.shape[1] * 2,
            transform=fixAff,
            crs=src.crs
            ) as dst:
            for i, b in enumerate(fixedArrays):
                dst.write_band(i + 1, b)

def smoosh_rasters(inputRasters, outputRaster, gfs, development):
    import rasterio

    rasInfo = list(gribdoctor.loadRasterInfo(b) for b in inputRasters)
    
    if abs(rasInfo[0]['affine'].c) > 360 and development == True:
        gfs = False
        kwargs = rasInfo[0]['kwargs']
    elif development == True:
        gfs = True

    snapShape = gribdoctor.getSnapDims(rasInfo)
    snapSrc = gribdoctor.getSnapAffine(rasInfo, snapShape)

    allBands = list(gribdoctor.loadBands(b, snapShape, gfs) for b in inputRasters)
    
    allBands = list(b for sub in allBands for b in sub)

    if gfs:
        zoomFactor = 2
        kwargs = gribdoctor.makeKwargs(allBands, snapSrc, snapShape, zoomFactor)
    else:
        zoomFactor = 1
        kwargs['count'] = len(allBands)
        kwargs['driver'] = 'GTiff'

    with rasterio.drivers():
        with rasterio.open(outputRaster, 'w', **kwargs) as dst:
            for i, b in enumerate(allBands):
                dst.write_band(i + 1, b)


import cut_splice_globewrap as csg, click

@click.group()
def cli():
    pass

@click.command('unwrap')
@click.argument('inputgrib', type=click.Path(exists=True))
@click.argument('output', type=click.Path(exists=False))
@click.option('--bidx', '-b', default='all',
    help='Bands to include [default = all]')
@click.option('--bandtags', '-bt', is_flag=True,
    help='Flag to indicate printing of band tags / band metadata to stdout')
def unwrap(inputgrib, output, bidx, bandtags):
    csg.upwrap_raster(inputgrib, output, bidx, bandtags)

cli.add_command(unwrap)

@click.command('smoosh')
@click.argument('inputs', type=click.Path(exists=True), nargs=-1)
@click.argument('output', type=click.Path(exists=False), nargs=1)
@click.option('--develoment', '-dev', is_flag=True,
    help='Dev tag for experimental features')
@click.option('--unwrap', '-uw', is_flag=True, default=True,
    help='Unwrap GFS Rasters')
def smoosh(inputs, output, develoment, unwrap):
    'stack multiple global gfs gribs of (possible) varying resolutions'
    csg.smoosh_rasters(inputs, output, unwrap, develoment)

cli.add_command(smoosh)